from collections import defaultdict
from itertools import combinations
from queue import Queue

COLORID = { 'W': 1, 'U': 2, 'B': 3, 'R': 4, 'G': 5, 'C': 6 }
            
BASICS = { 'Plains':'W', 'Island':'U', 'Swamp':'B', 'Mountain':'R', 'Forest':'G' }
THREECOLOR = { 'Arcane Sanctum':'W|U|B', 'Crumbling Necropolis':'U|B|R', 'Frontier Bivouac':'G|U|R', 'Jungle Shrine':'R|G|W', 'Mystic Monastery':'U|R|W', 'Nomad Outpost':'R|W|B', 'Opulent Palace':'B|G|U', 'Sandsteppe Citadel':'W|B|G', 'Savage Lands':'B|R|G', 'Seaside Citadel':'G|W|U' }
DUALS = { 'Badlands':'B|R', 'Bayou':'B|G', 'Plateau':'R|W', 'Savannah':'G|W', 'Scrubland':'W|B', 'Taiga':'R|G', 'Tropical Island':'G|U', 'Tundra':'W|U', 'Underground Sea':'U|B', 'Volcanic Island':'U|R' }
FETCHES = { 'Arid Mesa':'R|W', 'Bloodstained Mire':'B|R', 'Flooded Strand':'W|U', 'Marsh Flats':'W|B', 'Misty Rainforest':'G|U', 'Polluted Delta':'U|B', 'Scalding Tarn':'U|R', 'Verdant Catacombs':'B|G', 'Windswept Heath':'G|W', 'Wooded Foothills':'R|G' }
SHOCKS = { 'Blood Crypt':'B|R', 'Breeding Pool':'G|U', 'Godless Shrine':'W|B', 'Hallowed Fountain':'W|U', 'Overgrown Tomb':'B|G', 'Sacred Foundry':'R|W', 'Steam Vents':'U|R', 'Stomping Ground':'R|G', 'Temple Garden':'G|W', 'Watery Grave':'U|B' }
PAINS = { 'Adarkar Wastes':'W|U', 'Battlefield Forge':'R|W', 'Brushland':'G|W', 'Caves of Koilos':'W|B', 'Karplusan Forest':'R|G', 'Llanowar Wastes':'B|G', 'Shivan Reef':'U|R', 'Sulfurous Springs':'B|R', 'Underground River':'U|B', 'Yavimaya Coast':'G|U' }
BOUNCES = { 'Azorius Chancery':'WU', 'Boros Garrison':'RW', 'Dimir Aqueduct':'UB', 'Golgari Rot Farm':'BG', 'Gruul Turf':'RG', 'Izzet Boilerworks':'UR', 'Orzhov Basilica':'WB', 'Rakdos Carnarium':'BR', 'Selesnya Sanctuary':'GW', 'Simic Growth Chamber':'GU' }
TRON = { "Urza's Mine": 2, "Urza's Power Plant": 2, "Urza's Tower": 3 }
FIVECOLOR = { 'City of Brass': '-W|-U|-R|-B|-G', 'Gemstone Mine': 'W|U|R|B|G' }

EVERYTHING = { land : landtype[land] for landtype in [ BASICS, THREECOLOR, DUALS, FETCHES, SHOCKS, PAINS, BOUNCES, TRON, FIVECOLOR ] for land in landtype }

def can_fetch(fetch, fetchable):
    fetch_colors = EVERYTHING[fetch].split('|')
    fetchable_colors = EVERYTHING[fetchable].split('|')
    return any( x in fetch_colors for x in fetchable_colors )

class Edge:
    def __init__(self, src, dest, capacity, cost, residue):
        self.src = src
        self.dest = dest
        self.capacity = capacity
        self.cost = cost
        self.residue = residue

def add_arc(adjacency_lists, edge_list, src, dest, cap, cost):
    eid = len(edge_list)
    adjacency_lists[src].append( eid )
    adjacency_lists[dest].append( eid+1 )
    edge_list.append( Edge(  src, dest, cap,  cost, cap ) )
    edge_list.append( Edge( dest,  src, cap, -cost,   0 ) )

def mincost_maxflow(source, sink, adjacency_lists, edge_list):
    flow_size, flow_cost = 0, 0
    while True:
        incoming, distance = {}, defaultdict(lambda:10**9)
        distance[source] = 0
        for phase in range(len(adjacency_lists)-1):
            for where in adjacency_lists:
                if where in distance:
                    for edge_id in adjacency_lists[where]:
                        edge = edge_list[edge_id]
                        if edge.residue > 0:
                            if distance[edge.dest] > distance[where] + edge.cost:
                                distance[edge.dest] = distance[where] + edge.cost
                                incoming[edge.dest] = edge_id

        if sink not in incoming:
            return (flow_size,flow_cost)

        where = sink
        can_push = 2**30
        while where != source:
            edge = edge_list[ incoming[where] ]
            can_push = min( can_push, edge.residue )
            where = edge.src
        flow_size += can_push
        flow_cost += can_push * distance[sink]

        where = sink
        while where != source:
            edge_list[ incoming[where] ].residue -= can_push
            edge_list[ incoming[where]^1 ].residue += can_push
            where = edge_list[ incoming[where] ].src

def load():
    input()

    n1 = int( input() )
    IN_PLAY = defaultdict(int)
    for i in range(n1): IN_PLAY[ input() ] += 1

    n2 = int( input() )
    IN_DECK = defaultdict(int)
    for i in range(n2): IN_DECK[ input() ] += 1

    LIFE = int( input() )

    GOAL = [ 0, 0, 0, 0, 0, 0, 0 ]
    for x in input():
        if x.isdigit():
            GOAL[0] += int(x)
        else:
            GOAL[ COLORID[x] ] += 1

    return (IN_PLAY, IN_DECK, LIFE, GOAL)

def solve(IN_PLAY, IN_DECK, LIFE, GOAL):
    # the only special case: handle Tron
    has_tron = all( land in IN_PLAY for land in TRON )
    for land in TRON:
        if land in IN_PLAY:
            GOAL[ COLORID['C'] ] -= ( TRON[land] if has_tron else 1 ) * IN_PLAY[land]
            del IN_PLAY[land]
    if GOAL[ COLORID['C'] ] < 0:
        GOAL[0] = max(0, GOAL[0] + GOAL[ COLORID['C'] ] )
        GOAL[ COLORID['C'] ] = 0

    # then build the flow network
    adjacency_lists = defaultdict(list)
    edge_list = []

    # fixed node numbers:
    # 0 = source, 1 = sink, 23456 = colors, 7 = colorless, 8 = generic, 
    # edges for colors -> sink and colors -> generic -> sink
    for c in 'WUBRGC':
        add_arc( adjacency_lists, edge_list, 1+COLORID[c], 1, GOAL[ COLORID[c] ], 0 )
        add_arc( adjacency_lists, edge_list, 1+COLORID[c], 8, 2**30, 0 )
    add_arc( adjacency_lists, edge_list, 8, 1, GOAL[0], 0 )

    # starting from id = 8 we have nodes for individual lands, skipping fetch lands and fetchable lands for the moment
    nextid = 9 

    for land in IN_PLAY:
        for landtype in [ BASICS, THREECOLOR, DUALS, SHOCKS, {'Gemstone Mine':'W|U|B|G|R'} ]:
            if land in landtype:
                add_arc( adjacency_lists, edge_list, 0, nextid, IN_PLAY[land], 0 )
                for color in landtype[land].split('|'):
                    add_arc( adjacency_lists, edge_list, nextid, 1+COLORID[color], IN_PLAY[land], 0 )
                nextid += 1

        if land == 'City of Brass':
            add_arc( adjacency_lists, edge_list, 0, nextid, IN_PLAY[land], 0 )
            for color in 'WUBRG':
                add_arc( adjacency_lists, edge_list, nextid, 1+COLORID[color], IN_PLAY[land], 1 )
            nextid += 1

        if land in BOUNCES:
            add_arc( adjacency_lists, edge_list, 0, nextid, 2*IN_PLAY[land], 0 )
            for color in BOUNCES[land]:
                add_arc( adjacency_lists, edge_list, nextid, 1+COLORID[color], IN_PLAY[land], 0 )
            nextid += 1

        if land in PAINS:
            add_arc( adjacency_lists, edge_list, 0, nextid, IN_PLAY[land], 0 )
            add_arc( adjacency_lists, edge_list, nextid, 7, IN_PLAY[land], 0 )
            for color in PAINS[land].split('|'):
                add_arc( adjacency_lists, edge_list, nextid, 1+COLORID[color], IN_PLAY[land], 1 )
            nextid += 1

    # assign vertex IDs to fetch lands, add edges source -> fetch
    fetch_id = {}
    for land in IN_PLAY:
        if land in FETCHES:
            fetch_id[ land ] = nextid
            add_arc( adjacency_lists, edge_list, 0, nextid, IN_PLAY[land], 1 ) # going to fetch already costs a life
            nextid += 1

    # assign vertex IDs to fetchable lands, add edges fetchable -> colors
    fetchable_id = {}
    for land in IN_DECK:
        if not (land in BASICS or land in DUALS or land in SHOCKS):
            continue
        fetchable_id[land] = nextid
        add_arc( adjacency_lists, edge_list, nextid, nextid+1, IN_DECK[land], 0 ) # make sure it cannot be fetched too many times
        for color in EVERYTHING[land].split('|'):
            add_arc( adjacency_lists, edge_list, nextid+1, 1+COLORID[color], IN_DECK[land], 2 if land in SHOCKS else 0 )
        nextid += 2

    # add edges between fetch lands and fetchable lands
    for land1 in fetch_id:
        for land2 in fetchable_id:
            if can_fetch(land1,land2):
                add_arc( adjacency_lists, edge_list, fetch_id[land1], fetchable_id[land2], 2**30, 0 )

    # compute maxflow to check everything
    flow_size, flow_cost = mincost_maxflow( 0, 1, adjacency_lists, edge_list )
    return flow_size == sum(GOAL) and flow_cost < LIFE

# main
test_count = int( input() )
for test in range(test_count): print( 'YES' if solve(*load()) else 'NO' )

